'''Items
'''

import pyglet
import pymunk
import misc
import spritesheet
from spritesheet import spritesheet

class Item(object):
    def __init__(self, g, x = 0, y = 0, type = misc.ITEM_GENERIC):
        mass = 0.05
        img = spritesheet['potion'][39]
        img.anchor_x = img.width // 2
        img.anchor_y = img.height // 2
        self.sprite = pyglet.sprite.Sprite(img, x = x, y = y)
        radius = img.width // 2
        moment = pymunk.moment_for_circle(mass, 0, radius, (0,0))
        self.body = pymunk.Body(mass, moment)
        self.body.position.x = x
        self.body.position.y = y
        g.space.add(self.body)
        self.shape = pymunk.Circle(self.body, radius, (0,0))
        self.shape.collision_type = misc.COLLTYPE_ITEM
        g.space.add(self.shape)
        self.x = x
        self.y = y
        self.g = g
        self.type = type
        self.state = misc.STATE_NORMAL

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
        if name == 'x':
            self.sprite.x = value
            self.body.position.x = value
        if name == 'y':
            self.sprite.y = value
            self.body.position.y = value

    def __del__(self):
        if self.state != misc.STATE_GHOST:
            self.g.space.remove(self.body, self.shape)
            self.state = misc.STATE_GHOST

    def post_phystep(self):
        self.x = self.body.position.x
        self.y = self.body.position.y
        if self.state == misc.STATE_REAP:
            self.g.itemlist.remove(self)

    def draw(self):
        self.sprite.draw()


def collision(a, b, contacts, normal, g):
    '''Collision function for collision between player and item
    '''
    if a.collision_type == misc.COLLTYPE_ITEM:
        itemshape = a
    elif b.collision_type == misc.COLLTYPE_ITEM:
        itemshape = b
    else:
        # this should never happen
        print "didn't find item"
        return True
    targetitem = 0
    for item in g.itemlist:
        if item.shape == itemshape:
            targetitem = item
    if targetitem:
        if targetitem.state == misc.STATE_NORMAL:
            if g.inventory.slots_left() > 0:
                g.inventory.add_item(targetitem.type, targetitem.sprite.image)
                targetitem.state = misc.STATE_REAP
                g.gui.printconsole("Picked up a yummy!")
    return True

    
