'''Level loading
'''

import pyglet
import pymunk
import mob
import staticgeom
import item
import misc
import monster
import player
import combat
import exitdoor

def load_level(g, mapname = 'map1'):
    '''Load a level
    '''

    # set up combat
    g.arena = combat.Arena()

    # set up physics
    g.space = pymunk.Space()
    g.space.damping = 0.03
    g.space.add(g.mouse.body)
    g.space.add_collisionpair_func(misc.COLLTYPE_PLAYER, misc.COLLTYPE_ITEM,
                                   item.collision, data = g)
    g.space.add_collisionpair_func(misc.COLLTYPE_PLAYER, misc.COLLTYPE_MONSTER,
                                   monster.collision, data = g)
    g.space.add_collisionpair_func(misc.COLLTYPE_PLAYER, misc.COLLTYPE_EXIT,
                                   exitdoor.collision, data = g)

    # set up level data structures
    g.staticgeom = []
    g.itemlist = []
    g.monsterlist = []
    g.exitlist = []

    player_loaded = 0
    
    # load map from file into array
    mapary = []
    mapary.append([])
    i = 0
    j = 0
    width = 100
    height = 100
    filename = mapname + ".CSV"
    f = pyglet.resource.file(filename)
    for line in f.readlines():
        tilelist = line.rstrip().rstrip(',').split(', ')
        if i + len(tilelist) > width:
            print "FATAL: map format seems to be wrong"
            exit
        tilenums = [ int(tile) for tile in tilelist ]
        #print "appending %d to line %d" % (len(tilenums),j)
        mapary[j].extend(tilenums)
        i += len(tilenums)
        if i == width and j < height - 1:
            mapary.append([])
            j += 1
            i = 0
    #print repr(mapary)

    # Parse map array and create level objects
    scale = 100
    for j in range(height):
        for i in range(width):
            x = i * scale
            y = 0 - j * scale
            if mapary[j][i] == misc.TILE_WALLSTART:
                #print "brick start at %d,%d" % (i,j)
                # get horizontal endpoint, then vertical one
                # tile type TILE_WALLSTART marks lower left corner
                brick_tile = mapary[j][i+1]
                brick_width = 1
                ii = i+1
                while mapary[j][ii] == brick_tile:
                    brick_width += 1
                    ii += 1
                brick_height = 1
                jj = j-1
                while mapary[jj][i] == brick_tile:
                    brick_height += 1
                    jj -= 1
                #print "found brick: %d,%d and dim %d,%d" % (i,j,brick_width,
                #                                            brick_height)
                x1 = i * scale
                y1 = 0 - j * scale
                x2 = x1 + brick_width * scale
                y2 = y1 + brick_height * scale
                #print "making wall: %d,%d  %d,%d" % (x1,y1,x2,y2)
                g.staticgeom.append(staticgeom.Wall(g.space, x1 = x1, y1 = y1,
                                                    x2 = x2, y2 = y2))

            elif mapary[j][i] == misc.TILE_POTION:
                g.itemlist.append(item.Item(g, x = x + 50, y = y + 50))
            elif mapary[j][i] == misc.TILE_MONSTER:
                g.monsterlist.append(monster.Monster(g, x = x + 50, y = y + 50))
            elif mapary[j][i] == misc.TILE_SNAKESTART:
                if player_loaded:
                    print "WARNING: duplicate snake start tile!"
                else:
                    # set up player
                    g.player = player.Player(g, length = 7, x = x, y = y)
            elif mapary[j][i] == misc.TILE_EXIT:
                g.exitlist.append(exitdoor.ExitDoor(g, x = x, y = y))
