'''Miscellaneous objects and functions

These didn't get big enough to put in their own files.
'''

import pyglet
from pyglet.gl import *
import pymunk
import spritesheet
from spritesheet import spritesheet

# Various constants
COLLTYPE_DEFAULT = 0
COLLTYPE_PLAYER = 1
COLLTYPE_ITEM = 2
COLLTYPE_MONSTER = 3
COLLTYPE_EXIT = 4

STATE_NORMAL = 0
STATE_REAP = 1
STATE_GHOST = 2

TILE_WALLSTART = 2
TILE_POTION = 7
TILE_MONSTER = 8
TILE_SNAKESTART = 9
TILE_EXIT = 10

ITEM_GENERIC = 0


class GenericObject(object):
    '''Generic object, used mainly to store data and references.
    '''
    def __init__(self):
        pass


def create_start_screen(g):
    '''Creates objects needed to display the start screen.  This function
    must be called before starting the render loop (pyglet.app.run).
    '''
    g.startscreen = GenericObject()
    g.startscreen.labels = [
        pyglet.text.Label(text = 'Snake RPG', font_size = 36, bold = True,
                          color = (255, 255, 255, 255), x = 0, y = 300,
                          anchor_x = 'center'),
        pyglet.sprite.Sprite(img = \
                             pyglet.resource.image('small-red-snake.png'),
                             x = 0, y = 200),
        pyglet.text.Label(text = \
                          "Click and hold left button to move, scroll wheel to zoom",
                          font_size = 18,
                          x = 0, y = 100, anchor_x = 'center'),
        pyglet.sprite.Sprite(img = spritesheet['doorwall'][24],
                             x = -120, y = -50 ),
        pyglet.text.Label(text = "Find the door to WIN!",
                          font_size = 18,
                          x = 0, y = -25, anchor_y = 'bottom'),
        pyglet.text.Label(text = 'Press any key to start!', font_size = 24,
                          x = 0, y = -300, anchor_x = 'center')
        ]


def create_end_screen(g):
    g.endscreen = GenericObject()
    img = pyglet.resource.image('deathsnake.png')
    img.anchor_x = img.width // 2
    g.endscreen.labels = [
        pyglet.text.Label(text = 'OMG U TOTALLY DIED!', font_size = 48,
                          bold = True,
                          color = (255, 255, 255, 255), x = 0, y = 250,
                          anchor_x = 'center'),
        pyglet.sprite.Sprite(img = img, x = 0, y = 0),
        pyglet.text.Label(text = '[ Press any key ]', font_size = 24,
                          x = 0, y = -300, anchor_x = 'center')
        ]


def create_win_screen(g):
    g.winscreen = GenericObject()
    img = pyglet.resource.image('winner.png')
    img.anchor_x = img.width // 2
    img.anchor_y = img.height // 2
    g.winscreen.labels = [
        pyglet.text.Label(text = 'A WINNER IS YOU!!!', font_size = 64,
                          bold = True,
                          color = (255, 255, 255, 255), x = 0, y = 250,
                          anchor_x = 'center'),
        pyglet.sprite.Sprite(img = img, x = 0, y = 0),
        pyglet.text.Label(text = '[ Press any key ]', font_size = 24,
                          x = 0, y = -300, anchor_x = 'center')
        ]


def screen_to_world(x, y):
    '''Translate screen (mouse) coordinates to game world coordinates
    '''
    world_x, world_y, world_z = GLdouble(), GLdouble(), GLdouble()
    modelview , projection = (GLdouble * 16)(), (GLdouble * 16)()
    view = (GLint * 4)()
    glGetDoublev(GL_MODELVIEW_MATRIX,modelview)
    glGetDoublev(GL_PROJECTION_MATRIX,projection)
    glGetIntegerv(GL_VIEWPORT,view)
    wx, wy, wz = GLdouble(), GLdouble(), GLdouble()
    # Use point at center of game world
    gluProject(0.0, 0.0, 0.0, modelview, projection, view, wx, wy, wz)
    gluUnProject(x, y, wz, modelview,
                 projection, view, world_x, world_y, world_z)
    return int(world_x.value),int(world_y.value)


class Mouse(object):
    '''Mouse display and physics'''
    def __init__(self, x, y, cursor_size = 25):
        self.body = pymunk.Body(pymunk.inf, pymunk.inf)
        self.cursor_size = cursor_size
        self.x = x
        self.y = y
        self.button = 0

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
        if name == 'x':
            self.body.position.x = value
        if name == 'y':
            self.body.position.y = value

    def draw(self):
        '''Draw mouse cursor
        '''
        if self.button:
            glColor3f(1.0,0.3,0.5)
        else:
            glColor3f(1.0,1.0,0.0)
        glLineWidth(2)
        glBegin(GL_LINES)
        glVertex3f(self.x - self.cursor_size, self.y, 0.0)
        glVertex3f(self.x + self.cursor_size, self.y, 0.0)
        glVertex3f(self.x, self.y - self.cursor_size, 0.0)
        glVertex3f(self.x, self.y + self.cursor_size, 0.0)
        glEnd()
        glLineWidth(1)
