'''Mobs
'''

import pyglet
import pymunk
import math
import misc

class Mob(object):
    def __init__(self, g, x = 0, y = 0, spritefile = 'default_sprite.png',
                 mass = 1.0, img = None):
        if img == None:
            img = pyglet.resource.image(spritefile)
            img.anchor_x = img.width // 2
            img.anchor_y = img.height // 2
        self.sprite = pyglet.sprite.Sprite(img, x, y)
        radius = img.width // 2
        if mass < pymunk.inf:
            moment = pymunk.moment_for_circle(mass, 0, radius, (0,0))
        else:
            moment = pymunk.inf
        self.body = pymunk.Body(mass, moment)
        self.body.position.x = x
        self.body.position.y = y
        g.space.add(self.body)
        self.shape = pymunk.Circle(self.body, radius, (0,0))
        self.shape.collision_type = misc.COLLTYPE_MONSTER
        g.space.add(self.shape)
        self.x = x
        self.y = y
        self.g = g
        self.state = misc.STATE_NORMAL

    def __del__(self):
        if self.state != misc.STATE_GHOST:
            self.g.space.remove(self.body, self.shape)
            self.state = misc.STATE_GHOST

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
        if name == 'x':
            self.sprite.x = value
            self.body.position.x = value
        if name == 'y':
            self.sprite.y = value
            self.body.position.y = value

    def post_phystep(self):
        '''Update sprite positions to pymunk body positions'''
        self.x = self.body.position.x
        self.y = self.body.position.y

    def move_to(self, x, y):
        '''Move to the given position.'''
        self.x = x
        self.y = y

    def reset_forces(self):
        '''Reset any forces applied by other methods'''
        pass

    def draw(self):
        self.sprite.draw()
                                   
                                  
            
